#include <u.h>
#include <libc.h>
#include <bio.h>
#include "json.h"

static void freemap(Jmap *m);
static void freearray(Jarray *a);


static void
freevalue(Jvalue *v)
{
	if(v == nil)
		return;

	switch(v->type){
	case Tnull:
	case Ttrue:
	case Tfalse:
		break;
	case Tnumber:
		break;
	case Tstring:
		free(v->str);
		break;
	case Tmap:
		freemap(v->map);
		break;
	case Tarray:
		freearray(v->array);
		break;
	default:
		sysfatal("json_free: %d unknown value type", v->type);
		break;
	}
	free(v);
}

static void
freearray(Jarray *a)
{
	int i;

	if(a){
		for(i = 0; i < a->used; i++)
			freevalue(a->base[i]);
		free(a->base);
		free(a);
	}
}

static void
freemap(Jmap *map)
{
	Jmap *m, *t;

	for(m = map; m; m = t){
		freevalue(m->value);
		t = m->next;
		free(m->name);
		free(m);
	}
}

void
json_free(Jvalue *v)
{
	switch(v->type){
	case Tmap:
		freemap(v->map);
		break;
	case Tarray:
		freearray(v->array);
		break;
	case Tempty:
		break;
	default:
		sysfatal("json_free: %d unknown object type\n", v->type);
		break;
	}
	free(v);
}
