#include <u.h>
#include <libc.h>
#include <ctype.h>
#include "json.h"

/*
 * seperator between object names we use to parse
 * a json data structure.
 */
enum {
	Sep = '/'
};

static char *valarray(Jarray *a, char *path);
static char *valmap(Jmap *map, char *path);

static char *
value(Jvalue *v, char *path)
{
	static char buf[16];

	switch(v->type){
	case Tnull:
		return nil;
	case Ttrue:
		return "true";
	case Tfalse:
		return "false";
	case Tnumber:
		if(floor(v->num) == v->num)
			snprint(buf, sizeof(buf), "%.0f", v->num);
		else
			snprint(buf, sizeof(buf), "%g", v->num);
		return buf;
	case Tstring:
		return v->str;
	case Tmap:
		return valmap(v->map, path);
	case Tarray:
		return valarray(v->array, path);
	default:
		break;
	}
	abort();
	return nil;
}

static char *
pathnext(char *s, char *t, int isarray)
{
	while(*t && *s == *t){
		s++;
		t++;
	}
	
	if(*t)
		return nil;		/* no match */
	if(!isarray && *s == 0)
		return s;		/* match last map name of path */
	if(isarray && *s == '[')
		return s;		/* match an array */
	if(! isarray && *s == Sep)
		return s+1;		/* match map name */
	return nil;			/* partial match */
}

static char *
valarray(Jarray *a, char *path)
{
	int i, idx;
	char *p;

	for(p = path+1; *p && *p != Sep && *p != '['; p++)
		continue;
	if(*p == Sep)
		p++;

	if(*path != '[')
		sysfatal("valarray: %.*s - bad array index 1", (int)(p-path), path);
	idx = strtol(path+1, &path, 0);
	if(*path != ']')
		sysfatal("valarray: %.*s - bad array index 2", (int)(p-path), path);

	for(i = 0; a && i < a->used; i++){
		if(i == idx)
			return value(a->base[i], p);
	}
	return nil;
}

static char *
valmap(Jmap *map, char *path)
{
	Jmap *m;
	char *p;
	int isarray;

	for(m = map; m; m = m->next){
		isarray = 0;
		if(m->value && m->value->type == Tarray)
			isarray = 1;

		if((p = pathnext(path, m->name, isarray)) != nil)
			return value(m->value, p);
	}
	return nil;
}

char *
json_strval(Jvalue *v, char *fmt, ...)
{
	va_list ap;
	char *rc, path[1024];

	va_start(ap, fmt);
	vsnprint(path, sizeof(path), fmt, ap);
	va_end(ap);

	rc = nil;
	switch(v->type){
	case Tmap:
		rc = valmap(v->map, path);
		break;
	case Tarray:
		rc = valarray(v->array, path);
		break;
	default:
		break;
	}

	return rc;
}

